@@ -31,6 +31,12 @@ local scopeinfo  = require("base/scopeinfo")
 local deprecated = require("base/deprecated")
 local sandbox    = require("sandbox/sandbox")
 
+-- raise without interpreter stack
+-- @see https://github.com/xmake-io/xmake/issues/3553
+function interpreter._raise(errors)
+    os.raise("[nobacktrace]: " .. (errors or ""))
+end
+
 -- traceback
 function interpreter._traceback(errors)
 
@@ -76,8 +82,6 @@ function interpreter._traceback(errors)
         -- next
         level = level + 1
     end
-
-    -- ok?
     return results
 end
 
@@ -88,7 +92,7 @@ function interpreter._merge_root_scope(root, root_prev, override)
     root_prev = root_prev or {}
     for scope_kind_and_name, _ in pairs(root or {}) do
         -- only merge sub-scope for each kind("target@@xxxx") or __rootkind
-        -- we need ignore the sub-root scope e.g. target{} after fetching root scope
+        -- we need to ignore the sub-root scope e.g. target{} after fetching root scope
         --
         if scope_kind_and_name:find("@@", 1, true) or scope_kind_and_name == "__rootkind" then
             local scope_values = root_prev[scope_kind_and_name] or {}
@@ -354,7 +358,8 @@ function interpreter:_api_translate_paths(values, apiname, infolevel)
     for _, p in ipairs(values) do
         if type(p) ~= "string" or #p == 0 then
             local sourceinfo = debug.getinfo(infolevel or 3, "Sl")
-            os.raise("%s(%s): invalid path value at %s:%d", apiname, tostring(p), sourceinfo.short_src or sourceinfo.source, sourceinfo.currentline)
+            interpreter._raise(string.format("%s(%s): invalid path value at %s:%d", apiname, tostring(p),
+                sourceinfo.short_src or sourceinfo.source, sourceinfo.currentline))
         end
         if not p:find("^%s-%$%(.-%)") and not path.is_absolute(p) then
             table.insert(results, path.relative(path.absolute(p, self:scriptdir()), self:rootdir()))
@@ -477,7 +482,7 @@ function interpreter:_handle(scope, deduplicate, enable_filter)
 
         -- filter values
         --
-        -- @note we need do filter before removing repeat values
+        -- @note we need to do filter before removing repeat values
         -- https://github.com/xmake-io/xmake/issues/1732
         if enable_filter then
             values = self:_filter(values)
@@ -680,8 +685,8 @@ function interpreter.new()
 
     -- register the builtin interfaces
     instance:api_register(nil, "includes",     interpreter.api_builtin_includes)
-    instance:api_register(nil, "add_subdirs",  interpreter.api_builtin_includes)
-    instance:api_register(nil, "add_subfiles", interpreter.api_builtin_includes)
+    instance:api_register(nil, "add_subdirs",  interpreter.api_builtin_add_subdirs)
+    instance:api_register(nil, "add_subfiles", interpreter.api_builtin_add_subfiles)
     instance:api_register(nil, "set_xmakever", interpreter.api_builtin_set_xmakever)
     instance:api_register(nil, "save_scope",   interpreter.api_builtin_save_scope)
     instance:api_register(nil, "restore_scope",interpreter.api_builtin_restore_scope)
@@ -1076,7 +1081,7 @@ function interpreter:api_register_set_values(scope_kind, ...)
             extra_config = nil
         end
 
-        -- @note we need mark table value as meta object to avoid wrap/unwrap
+        -- @note we need to mark table value as meta object to avoid wrap/unwrap
         -- if these values cannot be expanded, especially when there is only one value
         --
         -- e.g. set_shflags({"-Wl,-exported_symbols_list", exportfile}, {force = true, expand = false})
@@ -1126,7 +1131,7 @@ function interpreter:api_register_add_values(scope_kind, ...)
             extra_config = nil
         end
 
-        -- @note we need mark table value as meta object to avoid wrap/unwrap
+        -- @note we need to mark table value as meta object to avoid wrap/unwrap
         -- if these values cannot be expanded, especially when there is only one value
         --
         -- e.g. add_shflags({"-Wl,-exported_symbols_list", exportfile}, {force = true, expand = false})
@@ -1573,13 +1578,13 @@ function interpreter:api_builtin_set_xmakever(minver)
 
     -- no version
     if not minver then
-        os.raise("[nobacktrace]: set_xmakever(): no version!")
+        interpreter._raise("set_xmakever(): no version!")
     end
 
     -- parse minimum version
     local minvers = minver:split('.', {plain = true})
     if not minvers or #minvers ~= 3 then
-        os.raise("[nobacktrace]: set_xmakever(\"%s\"): invalid version format!", minver)
+        interpreter._raise(string.format("set_xmakever(\"%s\"): invalid version format!", minver))
     end
 
     -- make minimum numerical version
@@ -1593,7 +1598,7 @@ function interpreter:api_builtin_set_xmakever(minver)
 
     -- check version
     if curvers_num < minvers_num then
-        os.raise("[nobacktrace]: xmake v%s < v%s, please run `$xmake update` to upgrade xmake!", xmake._VERSION_SHORT, minver)
+        interpreter._raise(string.format("xmake v%s < v%s, please run `$xmake update` to upgrade xmake!", xmake._VERSION_SHORT, minver))
     end
 end
 
@@ -1676,7 +1681,7 @@ function interpreter:api_builtin_includes(...)
                 -- done interpreter
                 local ok, errors = xpcall(script, interpreter._traceback)
                 if not ok then
-                    os.raise(errors)
+                    interpreter._raise(errors)
                 end
 
                 -- leave the script directory
@@ -1699,7 +1704,7 @@ function interpreter:api_builtin_includes(...)
                 -- get mtime of the file
                 self._PRIVATE._MTIMES[path.relative(file, self._PRIVATE._ROOTDIR)] = os.mtime(file)
             else
-                os.raise(errors)
+                interpreter._raise(errors)
             end
         end
     end
@@ -1708,6 +1713,20 @@ function interpreter:api_builtin_includes(...)
     self._PRIVATE._CURFILE = curfile
 end
 
+-- the builtin api: add_subdirs(), deprecated
+function interpreter:api_builtin_add_subdirs(...)
+    self:api_builtin_includes(...)
+    local dirs = {...}
+    deprecated.add("includes(%s)", "add_subdirs(%s)", table.concat(dirs, ", "), table.concat(dirs, ", "))
+end
+
+-- the builtin api: add_subfiles(), deprecated
+function interpreter:api_builtin_add_subfiles(...)
+    self:api_builtin_includes(...)
+    local files = {...}
+    deprecated.add("includes(%s)", "add_subfiles(%s)", table.concat(files, ", "), table.concat(files, ", "))
+end
+
 -- the builtin api: save_scope()
 -- save the current scope
 function interpreter:api_builtin_save_scope()
