@@ -878,6 +878,11 @@ function _instance:is_enabled()
     return self:get("enabled") ~= false
 end
 
+-- is rebuilt?
+function _instance:is_rebuilt()
+    return self:data("rebuilt")
+end
+
 -- get the enabled option
 function _instance:opt(name)
     return self:opts()[name]
@@ -1609,10 +1614,27 @@ function _instance:objectfiles()
     local deduplicate = batchcount > 1
 
     -- get object files from all dependent targets (object kind)
-    if self:orderdeps() then
-        for _, dep in ipairs(self:orderdeps()) do
-            if dep:kind() == "object" then
-                table.join2(objectfiles, dep:objectfiles())
+    -- @note we only merge objects in plain deps, e.g. binary -> (static -> object, object ...)
+    local plaindeps = self:get("deps")
+    if plaindeps and (self:is_binary() or self:is_shared() or self:is_static()) then
+        local function _get_all_objectfiles_of_object_dep (t)
+            local _objectfiles = {}
+            table.join2(_objectfiles, t:objectfiles())
+            local _plaindeps = t:get("deps")
+            if _plaindeps then
+                for _, depname in ipairs(_plaindeps) do
+                    local dep = t:dep(depname)
+                    if dep and dep:is_object() then
+                        table.join2(_objectfiles, _get_all_objectfiles_of_object_dep(dep))
+                    end
+                end
+            end
+            return _objectfiles
+        end
+        for _, depname in ipairs(table.wrap(plaindeps)) do
+            local dep = self:dep(depname)
+            if dep and dep:is_object() then
+                table.join2(objectfiles, _get_all_objectfiles_of_object_dep(dep))
                 deduplicate = true
             end
         end
@@ -1756,6 +1778,11 @@ function _instance:installfiles(outputdir)
     return self:_copiedfiles("installfiles", outputdir or self:installdir())
 end
 
+-- get the extra files
+function _instance:extrafiles()
+    return (self:_copiedfiles("extrafiles"))
+end
+
 -- get depend file from object file
 function _instance:dependfile(objectfile)
 
