@@ -39,6 +39,10 @@ sandbox_os.host         = os.host
 sandbox_os.arch         = os.arch
 sandbox_os.subhost      = os.subhost
 sandbox_os.subarch      = os.subarch
+sandbox_os.is_host      = os.is_host
+sandbox_os.is_arch      = os.is_arch
+sandbox_os.is_subhost   = os.is_subhost
+sandbox_os.is_subarch   = os.is_subarch
 sandbox_os.syserror     = os.syserror
 sandbox_os.strerror     = os.strerror
 sandbox_os.exit         = os.exit
@@ -119,11 +123,11 @@ function sandbox_os.rm(filepath)
 end
 
 -- link file or directory to the new symfile
-function sandbox_os.ln(srcpath, dstpath)
+function sandbox_os.ln(srcpath, dstpath, opt)
     assert(srcpath and dstpath)
     srcpath = tostring(srcpath)
     dstpath = tostring(dstpath)
-    local ok, errors = os.ln(vformat(srcpath), vformat(dstpath))
+    local ok, errors = os.ln(vformat(srcpath), vformat(dstpath), opt)
     if not ok then
         os.raise(errors)
     end
@@ -139,12 +143,12 @@ function sandbox_os.vcp(srcpath, dstpath, opt)
 end
 
 -- move file or directory with the verbose info
-function sandbox_os.vmv(srcpath, dstpath)
+function sandbox_os.vmv(srcpath, dstpath, opt)
     assert(srcpath and dstpath)
     if option.get("verbose") then
         utils.cprint("${dim}> move %s to %s", srcpath, dstpath)
     end
-    return sandbox_os.mv(srcpath, dstpath)
+    return sandbox_os.mv(srcpath, dstpath, opt)
 end
 
 -- remove file or directory with the verbose info
