@@ -202,7 +202,7 @@ function _fix_paths_for_precompiled_package(package)
             for _, file in ipairs(os.files(filepattern)) do
                 if remote_prefix then
                     local _, count = io.replace(file, remote_prefix, local_prefix, {plain = true})
-                    -- maybe we need translate path seperator
+                    -- maybe we need to translate path seperator
                     -- @see https://github.com/xmake-io/xmake/discussions/3008
                     if count == 0 and is_host("windows") then
                         io.replace(file, (remote_prefix:gsub("\\", "/")), local_prefix:gsub("\\", "/"), {plain = true})
@@ -246,8 +246,8 @@ function _clear_sourcedir(package)
     end
 end
 
--- install the given package
-function main(package)
+-- enter working directory
+function _enter_workdir(package)
 
     -- get working directory of this package
     local workdir = package:cachedir()
@@ -255,7 +255,7 @@ function main(package)
     -- lock this package
     package:lock()
 
-    -- enter the working directory
+    -- enter directory
     local oldir = nil
     local sourcedir = package:sourcedir()
     if sourcedir then
@@ -275,6 +275,47 @@ function main(package)
         oldir = os.cd(workdir)
     end
 
+    -- we need to copy source codes to the working directory with short path on windows
+    --
+    -- Because the target name and source file path of this project are too long,
+    -- it's absolute path exceeds the windows path length limit.
+    --
+    if is_host("windows") and package:policy("platform.longpaths") then
+        local sourcedir_tmp = os.tmpdir() .. ".dir"
+        os.tryrm(sourcedir_tmp)
+        os.cp(os.curdir(), sourcedir_tmp)
+        os.cd(sourcedir_tmp)
+    end
+
+    return oldir
+end
+
+-- leave working directory
+function _leave_workdir(package, oldir)
+
+    -- clean the empty package directory
+    local installdir = package:installdir()
+    if os.emptydir(installdir) then
+        os.tryrm(installdir)
+    end
+
+    -- unlock this package
+    package:unlock()
+
+    -- leave source codes directory
+    if oldir then
+        os.cd(oldir)
+    end
+
+    -- clean source directory if it is no longer needed
+    _clear_sourcedir(package)
+end
+
+function main(package)
+
+    -- enter working directory
+    local oldir = _enter_workdir(package)
+
     -- init tipname
     local tipname = package:name()
     if package:version_str() then
@@ -324,8 +365,8 @@ function main(package)
                     end
 
                     -- install rules
-                    local rulesdir = path.join(package:scriptdir(), "rules")
-                    if os.isdir(rulesdir) then
+                    local rulesdir = package:rulesdir()
+                    if rulesdir and os.isdir(rulesdir) then
                         os.cp(rulesdir, package:installdir())
                     end
 
@@ -432,19 +473,6 @@ function main(package)
         }
     }
 
-    -- clean the empty package directory
-    local installdir = package:installdir()
-    if os.emptydir(installdir) then
-        os.tryrm(installdir)
-    end
-
-    -- unlock this package
-    package:unlock()
-
-    -- leave source codes directory
-    os.cd(oldir)
-
-    -- clean source directory if it is no longer needed
-    _clear_sourcedir(package)
+    _leave_workdir(package, oldir)
     return ok
 end
